﻿// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedAutoPropertyAccessor.Global
// ReSharper disable MemberCanBeProtected.Global

using anbs_cp.Enums;
using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/*
    История версий
    Обновлено 2024.03.28
    * Все поля Errors, Warnings и Infos объеденины в одно Messages
    * В ActionStateMessage добавлен параметр, определяющий тип: MessageType
    
    Обновлено 2024.03.26
    * Все интерфейсы объеденины в один класс ActionStateMessage
    * Интерфейс IActionState удалён
    
    Обновлено 2023.12.10
    * Заменены классы Action* на соответствующие интерфейсы
    * Класс ActionState теперь наследует интерфейс IActionState, позволяющий добавлять состояния (ошибки, предупреждения и информацию)
        из разных классов ActionState, т.е. например, данные из ActionState<bool> (кроме значения!) можно перенести в ActionState<int>.

    Обновлено 2023.11.1
    * Добавлен возвращаемый результат

    Обновлено 2023.01.121.1:
    * Заменены интерфейсы IAction* на соответствующие классы
*/

/// <summary>
/// Состояние действия
/// </summary>
/// <typeparam name="T">Тип класса</typeparam>
public class ActionState<T>: ISerializable
{
    /// <summary>
    /// Список информации
    /// </summary>
    // ReSharper disable once CollectionNeverQueried.Local
    internal List<ActionStateMessage> Messages { get; } = [];

    /// <summary>
    /// Значение
    /// </summary>
    public T? Value { get; set; }

    /// <summary>
    /// Метод для выбора всех значений для условия
    /// </summary>
    // ReSharper disable once StaticMemberInGenericType
    public static readonly Func<ActionStateMessage, bool> SelectAll = static _ => true;

    #region Методы
    #region Очистка
    /// <summary>
    /// Очищает список ошибок
    /// </summary>
    public void ClearErrors ()
    {
        Clear(static message => message.MessageType == EActionStateMessageType.Error);
    }

    /// <summary>
    /// Очищает список предупреждений
    /// </summary>
    public void ClearWarnings ()
    {
        Clear(static message => message.MessageType == EActionStateMessageType.Warning);
    }

    /// <summary>
    /// Очищает список информации
    /// </summary>
    public void ClearInfo ()
    {
        Clear(static message => message.MessageType == EActionStateMessageType.Information);
    }

    /// <summary>
    /// Очищает список по условию <paramref name="predicate"/>
    /// </summary>
    /// <param name="predicate">Условие выборки</param>
    public void Clear (Func<ActionStateMessage, bool> predicate)
    {
        // Получаю список элементов
        IEnumerable<ActionStateMessage> list = Messages.Where(predicate);

        // Для каждого элемента
        foreach (ActionStateMessage actionStateMessage in list)
            // - запускаю его удаление
            Messages.Remove(actionStateMessage);
    }
    #endregion

    #region Добавление
    /// <summary>
    /// Добавление сообщения
    /// </summary>
    /// <param name="message">Сообщение</param>
    // ReSharper disable once MemberCanBeMadeStatic.Global
    // ReSharper disable once FunctionRecursiveOnAllPaths
    public void Add (ActionStateMessage message) => Messages.Add(message);

    /// <summary>
    /// Добавляет список
    /// </summary>
    /// <param name="messages">Список сообщений</param>
    public void AddRange (IEnumerable<ActionStateMessage> messages) => Messages.AddRange(messages);

    #endregion

    #region Добавление ошибки
    /// <summary>
    /// Добавление ошибки
    /// </summary>
    /// <param name="critical">Является ли ошибка критической</param>
    public void AddError (bool critical = true) => Add(new(EActionStateMessageType.Error, critical));

    /// <summary>
    /// Добавление ошибки
    /// </summary>
    /// <param name="message">Сообщение об ошибке</param>
    /// <param name="critical">Является ли ошибка критической</param>
    public void AddError (string message, bool critical = true) =>
        Add(new(EActionStateMessageType.Error, critical, message: message));

    /// <summary>
    /// Добавление ошибки
    /// </summary>
    /// <param name="errorObject">Объект ошибки</param>
    /// <param name="message">Сообщение об ошибке</param>
    /// /// <param name="critical">Является ли ошибка критической</param>
    public void AddError (string errorObject, string message, bool critical = true) =>
        Add(new(EActionStateMessageType.Error, critical, errorObject, message));

    #endregion

    #region Добавление предупреждения
    /// <summary>
    /// Добавление предупреждение
    /// </summary>
    /// <param name="message">Текст предупреждения</param>
    /// <param name="warningObject">Объект предупреждения</param>
    public void AddWarning (string message, string warningObject = "") =>
        Add(new(EActionStateMessageType.Warning, message: message, obj: warningObject));
    #endregion

    #region Добавление информации
    /// <summary>
    /// Добавление информации
    /// </summary>
    /// <param name="message">Текст информации</param>
    /// <param name="infoObject">Объект информации</param>
    public void AddInfo (string message, string infoObject = "") => Add(new(message: message, obj: infoObject));
    #endregion

    #region Печать
    /// <summary>
    /// Печать сообщений
    /// </summary>
    /// <param name="area">Область печати</param>
    /// <param name="formatList">Оформление всей области</param>
    /// <param name="formatItem">Оформление элемента</param>
    /// <returns>Текстовое представление выбранных списокв</returns>
    /// <exception cref="ArgumentOutOfRangeException">Ошибка при неизвестной области</exception>
    public string PrintMessage (EActionStatePrintArea area, string formatList, string formatItem)
    {
        // Создаю список
        List<ActionStateMessage> list = [];

        // Выбираю списки по областям
        switch (area)
        {
            case EActionStatePrintArea.All:
                list.AddRange(Messages);
                break;
            case EActionStatePrintArea.ErrorsAndWarnings:
                list.AddRange(Messages.Where(static message =>
                    message.MessageType is EActionStateMessageType.Error or EActionStateMessageType.Warning));
                break;
            case EActionStatePrintArea.ErrorsOnly:
                list.AddRange(Messages.Where(static message => message.MessageType == EActionStateMessageType.Error));
                break;
            case EActionStatePrintArea.WarningsOnly:
                list.AddRange(Messages.Where(static message => message.MessageType == EActionStateMessageType.Warning));
                break;
            case EActionStatePrintArea.InfosOnly:
                list.AddRange(Messages.Where(static message =>
                    message.MessageType == EActionStateMessageType.Information));
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(area), area, null);
        }

        // Создаю элементы
        string elements =
            list.Aggregate(string.Empty, (current, item) => current + item.PrintMessage(formatItem));

        return string.Format(formatList, elements);
    }
    #endregion

    #region Проверка на наличие
    /// <summary>
    /// Проверяет, есть ли ошибки
    /// </summary>
    /// <param name="ignoreNonCritical">Игнорировать не критические</param>
    /// <returns>Наличие ошибок</returns>
    public bool HasErrors (bool ignoreNonCritical = false) => Messages.Any(message =>
        message.MessageType == EActionStateMessageType.Error && (!ignoreNonCritical || !message.IsCritical));

    /// <summary>
    /// Проверяет, есть ли предупреждения
    /// </summary>
    /// <param name="ignoreInformWarning">Игнорировать информационные предупреждения</param>
    /// <returns>Наличие предупреждений</returns>
    public bool HasWarnings (bool ignoreInformWarning = false) => Messages.Any(message =>
        message.MessageType == EActionStateMessageType.Warning && (!ignoreInformWarning || !message.IsCritical));

    /// <summary>
    /// Проверяет, есть ли сообщения
    /// </summary>
    /// <param name="ignoreStatus">Игнорировать статусные сообщения</param>
    /// <returns>Наличие сообщений</returns>
    public bool HasInfo (bool ignoreStatus) => Messages.Any(message =>
        message.MessageType == EActionStateMessageType.Information && (!ignoreStatus || !message.IsCritical));

    /// <summary>
    /// Успешно ли завершилось
    /// </summary>
    public bool IsSuccess (bool ignoreNonCriticalErrors = false) =>
        !HasErrors(ignoreNonCriticalErrors) && !HasWarnings(true);
    #endregion

    #region Количество сообщений
    /// <summary>
    /// Количество сообщений, удовлеьворяющих условию <paramref name="predicate"/>
    /// </summary>
    /// <param name="predicate">Условие выборки</param>
    /// <returns>Число сообщений</returns>
    public int Count (Func<ActionStateMessage, bool> predicate) => Messages.Count(predicate);

    /// <summary>
    /// Количество ошибок
    /// </summary>
    /// <param name="ignoreNonCritical">Игнорировать не критические</param>
    /// <returns>Количество ошибок</returns>
    public int ErrorsCount (bool ignoreNonCritical = false) => Count(message =>
        message.MessageType == EActionStateMessageType.Error && (!ignoreNonCritical || !message.IsCritical));

    /// <summary>
    /// Количество предупреждений
    /// </summary>
    /// <param name="ignoreInformWarning">Игнорировать информационные предупреждения</param>
    /// <returns>Количество предупреждений</returns>
    public int WarningsCount (bool ignoreInformWarning = false) => Count(message =>
        message.MessageType == EActionStateMessageType.Warning && (!ignoreInformWarning || !message.IsCritical));

    /// <summary>
    /// Количество информационных сообщений
    /// </summary>
    /// <param name="ignoreStatus">Игнорировать статусные сообщения</param>
    /// <returns>Количество информационных сообщений</returns>
    public int InfoCount (bool ignoreStatus) => Count(message =>
        message.MessageType == EActionStateMessageType.Information && (!ignoreStatus || !message.IsCritical));
    #endregion

    #region Добавление другого состояния

    /// <summary>
    /// Добавляет другое состояние (например, результат другого действия, который возвращает <see cref="ActionState"/>).
    /// </summary>
    /// <typeparam name="TO">Тип параметра</typeparam>
    /// <param name="state">Запись состояния</param>
    public void AddState<TO> (ActionState<TO> state) => AddRange(state.Messages);
    #endregion
    #endregion

    #region Реализация интерфейса ISerializable
    /// <inheritdoc />
    public string Serialize ()
    {
        // Создаю модель
        ActionStateSerializable serializableModel = new();

        // Для каждого сообщения
        foreach (ActionStateMessage message in Messages)
            // - добавляю в модель
            serializableModel.Messages.Add(message.Serialize());

        // Создаю модель значения
        ActionStateSerializableValue value = new();

        // Получаю данные
        value.GetValue(Value);

        // Добавляю в модель
        serializableModel.Value = value.Serialize();

        // Возвращаю сериализованную модель
        return new NewtonsoftJsonSerializer().Serialize(serializableModel);
    }

    /// <inheritdoc />
    public void Deserialize (string json)
    {
        // Удаляю лишние символы
        json = json.Trim("\"".ToCharArray());

        // Десериализую строку
        ActionStateSerializable itemSerializable =
            new NewtonsoftJsonSerializer().Deserialize<ActionStateSerializable>(json) ?? new();

        // Создаю модель значения
        ActionStateSerializableValue value = new();

        // Очищаю списки
        Clear(SelectAll);

        // Для каждого сообщения
        foreach (string messageString in itemSerializable.Messages)
        {
            // - создаю сообщение
            ActionStateMessage message = new();
            // - десериализую в него данные из модели
            message.Deserialize(messageString);
            // - добавляю в список
            Add(message);
        }

        // Десериализую данные значения из модели
        value.Deserialize(itemSerializable.Value);

        // Получаю значение
        Value = value.SetValue<T>();
    }
    #endregion
}