﻿// ReSharper disable MemberCanBeInternal
// ReSharper disable MemberCanBePrivate.Global

using anbs_cp.Enums;
using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/// <summary>
/// Класс сообщения состояния
/// </summary>
public sealed class ActionStateMessage (
    EActionStateMessageType type = EActionStateMessageType.Information,
    bool isCritical = false,
    string obj = "",
    string message = ""): ISerializable
{
    #region Свойства
    /// <summary>
    /// Тип сообщения
    /// </summary>
    public EActionStateMessageType MessageType { get; set; } = type;

    /// <summary>
    /// Критичность сообщения
    /// </summary>
    public bool IsCritical { get; set; } = isCritical;

    /// <summary>
    /// Объект сообщения
    /// </summary>
    public string Object { get; set; } = obj;

    /// <summary>
    /// Текст сообщения
    /// </summary>
    public string Message { get; set; } = message;

    #endregion

    #region Методы

    /// <summary>
    /// Вывод сообщения
    /// </summary>
    /// <param name="format">Строка-форматирование (например, «[{0}] - {1}»)</param>
    /// <returns>Отформатированную строка</returns>
    public string PrintMessage (string format) => string.Format(format, Object, Message);

    /// <summary>
    /// Устанавливает объект
    /// </summary>
    /// <param name="obj">Объект</param>
    /// <typeparam name="T">Тип объекта</typeparam>
    public void SetObject<T> (T obj)
    {
        // Если объект реализует интерфейс ISerializable
        if (obj is ISerializable serializable)
        {
            // - то сериализуем его методами интерфейса
            Object = serializable.Serialize();
            // - и прерываем
            return;
        }

        // Сериализуем объект с помощью NewtonsoftJson
        Object = new NewtonsoftJsonSerializer().Serialize(obj);
    }

    /// <summary>
    /// Получает объект
    /// </summary>
    /// <typeparam name="T">Класс объекта</typeparam>
    /// <returns>Объект или null</returns>
    public T? GetObject<T> () => new NewtonsoftJsonSerializer().Deserialize<T>(Object);

    /// <summary>
    /// Получает объект
    /// </summary>
    /// <typeparam name="T">Класс объекта, реализующий интерфейс ISerializable</typeparam>
    /// <returns>Объект</returns>
    public T GetSerializedObject<T> () where T : ISerializable, new()
    {
        // Создаём результирующую модель
        T model = new();

        // Десериализуем её методами интерфейса ISerializable
        model.Deserialize(Object);

        // Возвращаем модель
        return model;
    }

    #endregion

    #region Реализация интерфейса ISerializable

    /// <inheritdoc />
    public string Serialize () => new NewtonsoftJsonSerializer().Serialize(this);

    /// <inheritdoc />
    public void Deserialize (string json)
    {
        // Десериализую строку
        ActionStateMessage item = new NewtonsoftJsonSerializer().Deserialize<ActionStateMessage>(json) ?? new();

        // Передаю параметры
        MessageType = item.MessageType;
        IsCritical = item.IsCritical;
        Object = item.Object;
        Message = item.Message;
    }

    #endregion
}