﻿using anbs_cp.Exceptions;
using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/// <summary>
/// Состояние действия
///
/// Обновлено 2024.03.26
/// * Добавлен класс ActionState для классов, поддерживаемых интерфейс I
/// </summary>
public sealed class ActionStateSerialized<T>: ActionState<T> where T : ISerializable, new()
{
    /// <summary>
    /// Добавляет другое состояние (например, результат другого действия, который возвращает <see cref="ActionState"/>).
    /// </summary>
    /// <param name="state">Запись состояния</param>
    public void AddState (ActionStateSerialized<T> state)
    {
        base.AddState(state);
    }

    #region Переопределение реализации интерфейса ISerializable
    /// <summary>
    /// Сериализовать элемент в формат json
    /// </summary>
    /// <returns>Строка в формате json</returns>
    /// <exception cref="SerializeException">Ошибка при сериализации</exception>
    public new string Serialize ()
    {
        // Создаю модель
        ActionStateSerializable serializableModel = new();

        // Для каждого сообщения
        foreach (ActionStateMessage message in Messages)
            // - добавляю в модель
            serializableModel.Messages.Add(message.Serialize());

        // Создаю модель значения
        ActionStateSerializableValue value = new();

        // Получаю данные
        value.GetSerializedValue(Value);

        // Добавляю в модель
        serializableModel.Value = value.Serialize();

        // Возвращаю сериализованную модель
        return new SysTextSerializer().Serialize(serializableModel);
    }

    /// <summary>
    /// Восстановить элемент из формата json
    /// </summary>
    /// <param name="json">Строка в формате json</param>
    /// <exception cref="SerializeException">Ошибка при десериализации</exception>
    public new void Deserialize (string json)
    {
        // Десериализую строку
        ActionStateSerializable itemSerializable = new SysTextSerializer().Deserialize<ActionStateSerializable>(json) ?? new();

        // Создаю модель значения
        ActionStateSerializableValue value = new();

        // Очищаю списки
        Clear(SelectAll);

        // Для каждого сообщения
        foreach (string messageString in itemSerializable.Messages)
        {
            // - создаю сообщение
            ActionStateMessage message = new();
            // - десериализую в него данные из модели
            message.Deserialize(messageString);
            // - добавляю в список
            Add(message);
        }

        // Десериализую данные значения из модели
        value.Deserialize(itemSerializable.Value);

        // Получаю значение
        Value = value.SetSerializedValue<T>();
    }
    #endregion
}