﻿using anbs_cp.ForNet.Enums;

using Ganss.Xss;

namespace anbs_cp.ForNet.Classes;

/// <summary>
///   Очистка текста от лишних HTML-тегов
/// </summary>
public static class Sanitizer
{
  #region Свойства

  /// <summary>
  ///   Все теги запрещены
  /// </summary>
  public static SanitizerAllowedHtml AllowedNone => GetNone();

  /// <summary>
  ///   Все теги разрешены
  /// </summary>
  public static SanitizerAllowedHtml AllowedAll => GetAll();

  /// <summary>
  ///   Оставлены только текстовые теги
  /// </summary>
  public static SanitizerAllowedHtml AllowedTextOnly => GetTextFormatOnly();

  /// <summary>
  ///   Оставлены только текстовые теги, а также img и a
  /// </summary>
  public static SanitizerAllowedHtml AllowedImageAndLinks => GetImageAndLinks();

  /// <summary>
  ///   Применяются все теги, кроме iframe
  /// </summary>
  public static SanitizerAllowedHtml AllowedAllExceptIFrame => GetAllExceptIFrame();

  #endregion

  #region Методы

  /// <summary>
  ///   Очистка html-кода <paramref name="html" /> согласно параметрам <paramref name="allowedHtml" />
  /// </summary>
  /// <param name="html">HTML-код</param>
  /// <param name="allowedHtml">Параметры очистки</param>
  /// <returns>Очищенный html-кода</returns>
  public static string SanitizeHtml (string html, SanitizerAllowedHtml allowedHtml)
  {
    // Создаю очиститель
    HtmlSanitizer sanitizer = new()
    {
      // - сохраняю дочерние удалённых
      KeepChildNodes = true
    };

    // Выключаю все параметры HTML
    sanitizer.AllowedTags.Clear();
    sanitizer.AllowedSchemes.Clear();
    sanitizer.AllowedCssProperties.Clear();
    sanitizer.AllowedClasses.Clear();
    sanitizer.AllowedAttributes.Clear();
    sanitizer.AllowedAtRules.Clear();
    sanitizer.AllowDataAttributes = false;

    // Загружаю параметры
    sanitizer.AllowedTags.UnionWith(allowedHtml.AllowedTags);
    sanitizer.AllowedSchemes.UnionWith(allowedHtml.AllowedSchemes);
    sanitizer.AllowedCssProperties.UnionWith(allowedHtml.AllowedCssProperties);
    sanitizer.AllowedClasses.UnionWith(allowedHtml.AllowedClasses);
    sanitizer.AllowedAttributes.UnionWith(allowedHtml.AllowedAttributes);
    sanitizer.AllowDataAttributes = allowedHtml.AllowDataAttributes;

    // Очищаю html согласно правилам
    return sanitizer.Sanitize(html);
  }

  /// <summary>
  ///   Очистка html-кода по уровню очистки
  /// </summary>
  /// <param name="html">HTML-код</param>
  /// <param name="level">Уровень очистка</param>
  /// <returns>Очищенный html-код</returns>
  public static string SanitizeHtml (string html, ESanitizerLevel level)
  {
    // Получаю параметры очистки
    SanitizerAllowedHtml allowedHtml = level switch
    {
      ESanitizerLevel.NoTags => AllowedNone,
      ESanitizerLevel.TextFormatOnly => AllowedTextOnly,
      ESanitizerLevel.ImageAndLinks => AllowedImageAndLinks,
      ESanitizerLevel.AllExceptIFrame => AllowedAllExceptIFrame,
      ESanitizerLevel.All => AllowedAll,
      var _ => AllowedAll
    };

    // Очищаю код и возвращаю результат очистки
    return SanitizeHtml(html, allowedHtml);
  }

  #endregion

  #region Вспомогателдьные методы

  /// <summary>
  ///   Получаю параметры, удаляющие все теги
  /// </summary>
  /// <returns>Параметры очистки</returns>
  private static SanitizerAllowedHtml GetNone () =>
    new()
    {
      AllowedTags = [],
      AllowedAttributes = [],
      AllowedCssProperties = [],
      AllowedClasses = [],
      AllowedSchemes = [],
      AllowDataAttributes = false
    };

  /// <summary>
  ///   Получаю параметры по умолчанию (разрешающие все теги)
  /// </summary>
  /// <returns>Параметры очистки</returns>
  private static SanitizerAllowedHtml GetAll ()
  {
    // Создаю очиститель
    HtmlSanitizer sanitizer = new();

    // Создаю модель
    return new()
    {
      AllowedTags = sanitizer.AllowedTags.ToList(),
      AllowedAttributes = sanitizer.AllowedAttributes.ToList(),
      AllowedCssProperties = sanitizer.AllowedCssProperties.ToList(),
      AllowedClasses = sanitizer.AllowedClasses.ToList(),
      AllowedSchemes = sanitizer.AllowedSchemes.ToList(),
      AllowDataAttributes = true
    };
  }

  /// <summary>
  ///   Параметры, оставляющие только текстовые теги
  /// </summary>
  /// <returns>Параметры очистки</returns>
  private static SanitizerAllowedHtml GetTextFormatOnly () =>
    new()
    {
      AllowedTags =
      [
        "strong", "b", "em", "i", "u", "hr", "strike", "div", "ol", "ul", "li", "p", "span", "h1", "h2", "h3",
        "h4"
      ],

      // ReSharper disable StringLiteralTypo
      AllowedAttributes =
      [
        "align", "bgcolor", "border", "cellpadding", "cellspacing", "charset", "checked", "class", "clear",
        "color",
        "cols", "colspan", "datetime", "disabled", "headers", "height", "high", "hspace", "label", "lang",
        "list",
        "low", "max", "maxlength", "min", "name", "nowrap", "placeholder", "required", "rev", "rows", "rowspan",
        "rules", "selected", "size", "span", "spellcheck", "style", "summary", "tabindex", "title", "type",
        "valign",
        "value", "vspace", "width", "wrap"
      ]
      // ReSharper restore StringLiteralTypo
    };

  /// <summary>
  ///   Параметры, оставляющие только текстовые теги, а также img и a
  /// </summary>
  /// <returns>Параметры очистки</returns>
  private static SanitizerAllowedHtml GetImageAndLinks ()
  {
    // Получаю текстовые параметры
    SanitizerAllowedHtml result = AllowedTextOnly;

    // Добавляю теги
    result.AllowedTags.AddRange(["a", "img"]);

    // Добавляю параметры
    // ReSharper disable StringLiteralTypo
    result.AllowedAttributes.AddRange(["alt", "href", "hreflang", "nohref", "rel", "src", "target"]);
    // ReSharper restore StringLiteralTypo

    // Возвращаю результат
    return result;
  }

  /// <summary>
  ///   Применяются все теги, кроме iframe
  /// </summary>
  /// <returns>Параметры очистки</returns>
  private static SanitizerAllowedHtml GetAllExceptIFrame ()
  {
    // Получаю все параметры
    SanitizerAllowedHtml result = AllowedAll;

    // Удаляю iframe
    result.AllowedTags.Remove("iframe");

    // Возвращаю результат
    return result;
  }

  #endregion
}